package org.adammarblestone.fsksamplerv0;

import android.os.Bundle;
import android.app.Activity;
import android.view.Menu;
import android.media.*;
import java.io.*;
import android.os.Environment;
import android.util.Log;
import android.content.Context;
import android.widget.LinearLayout;

import com.jjoe64.graphview.BarGraphView;
import com.jjoe64.graphview.GraphView;
import com.jjoe64.graphview.GraphView.GraphViewData;
import com.jjoe64.graphview.GraphViewSeries;
import com.jjoe64.graphview.LineGraphView;

public class MainActivity extends Activity {

	public boolean isRecording;
	public String filename = "localAudioStorage";

	
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        
        // From http://eurodev.blogspot.com/2009/09/raw-audio-manipulation-in-android.html
        Thread thread = new Thread(new Runnable() {
        	public void run() {
        		record();
        	} 
        });

        isRecording = true; // added this
        
        thread.start();
        
        try {
        	// got an error here: object not locked by thread before wait()
        	// so switched to using synchronization
        	synchronized(this){
        		 this.wait(10000);
        	}        
        } catch (InterruptedException e) {}

        isRecording = false;

        try {
        	thread.join();
        } catch (InterruptedException e) {}

        play();
        
        //finish();
        
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.activity_main, menu);
        return true;
    }
    
    // From http://eurodev.blogspot.com/2009/09/raw-audio-manipulation-in-android.html
    public void record() {
    	int frequency = 44100; // changed this to avoid a -2 output from AudioRecord.getMinBufferSize
    	int channelConfiguration = AudioFormat.CHANNEL_IN_MONO;
    	int audioEncoding = AudioFormat.ENCODING_PCM_16BIT;
    	
    	// Got rid of the use of the SD card for storage and instead using local file storage
		Log.d("MainActivity", "Got to 1");
		
		String filePath = this.getFilesDir().getPath().toString() + filename;
    	File file = new File(filePath);
    	
		Log.d("MainActivity", "Got to 2");

    	// Delete any previous recording.
    	if (file.exists())
    		file.delete();
    	
    	try{
    		file.createNewFile();
    	} catch(IOException e){
    		Log.d("MainActivity", "Error creating file.");
    		e.printStackTrace();
    	}
    	 
		Log.d("MainActivity", "Got to 3");

    	try {
    		
    		// Create a DataOuputStream to write the audio data into the saved file.
    		OutputStream os = new FileOutputStream(file);
    		Log.d("MainActivity", "Got to 4");

    		BufferedOutputStream bos = new BufferedOutputStream(os);
    		DataOutputStream dos = new DataOutputStream(bos);
    		Log.d("MainActivity", "Got to 5");

    		// Create a new AudioRecord object to record the audio.
    		int bufferSize = AudioRecord.getMinBufferSize(frequency, channelConfiguration, audioEncoding);
    		Log.d("MainActivity", "Audio buffer size = " + bufferSize);

    		AudioRecord audioRecord = new AudioRecord(MediaRecorder.AudioSource.MIC, 
    				frequency, channelConfiguration, 
    				audioEncoding, bufferSize);
    		Log.d("MainActivity", "Got to 6");
    		
    		short[] buffer = new short[bufferSize]; 
    		audioRecord.startRecording();

    		while (isRecording) {
    			int bufferReadResult = audioRecord.read(buffer, 0, bufferSize);
    			for (int i = 0; i < bufferReadResult; i++)
    				dos.writeShort(buffer[i]);
    		}

    		audioRecord.stop();
    		audioRecord.release();
    		dos.close();

    	} catch (Throwable t) {
    		Log.e("AudioRecord","Recording Failed");
    		t.printStackTrace();

    	}
    }
    
    // From http://eurodev.blogspot.com/2009/09/raw-audio-manipulation-in-android.html
    public void play() {
    	// Get the file we want to playback.
    	// Changed this to local storage from SD card storage.
		String filePath = this.getFilesDir().getPath().toString() + filename;
    	File file = new File(filePath);    	
    	// Get the length of the audio stored in the file (16 bit so 2 bytes per short)
    	// and create a short array to store the recorded audio.
    	int musicLength = (int)(file.length()/2);
    	
		Log.d("MainActivity", "music length " + musicLength);
    	
    	short[] music = new short[musicLength];

    	try {
    		    		
    		// Create a DataInputStream to read the audio data back from the saved file.
    		InputStream is = new FileInputStream(file);
    		BufferedInputStream bis = new BufferedInputStream(is);
    		DataInputStream dis = new DataInputStream(bis);

    		// Read the file into the music array.
    		int i = 0;
    		while (dis.available() > 0) {
    			music[musicLength-1-i] = dis.readShort();
    			i++;
    		}

    		// Close the input streams.
    		dis.close(); 
    		Log.d("MainActivity", "Got to 7");
    		
            // Modified from SimpleGraph.java
    		GraphViewData[] g = new GraphViewData[musicLength/500];
    		for(i=0; i < musicLength/500; i++){
    			g[i] = new GraphViewData(i, (double) music[i]);
    		}    		
    		GraphView graphView = new LineGraphView(this, "Audio input");
    		GraphViewSeries data = new GraphViewSeries(g);
    		graphView.addSeries(data);
    		LinearLayout layout = (LinearLayout) findViewById(R.id.mainlayout);
    		Log.d("MainActivity", "Layout: " + layout.toString());    		
    		layout.addView(graphView);

    		// Create a new AudioTrack object using the same parameters as the AudioRecord
    		// object used to create the file.
    		AudioTrack audioTrack = new AudioTrack(AudioManager.STREAM_MUSIC, 
    				44100, // changed this
    			AudioFormat.CHANNEL_OUT_MONO, // changed this
    		AudioFormat.ENCODING_PCM_16BIT, 
    		musicLength, 
    		AudioTrack.MODE_STREAM);
    		
    		Log.d("MainActivity", "Got to 8");
    		
    		// Start playback
    		audioTrack.play();
    		Log.d("MainActivity", "Got to 9");

    		// Write the music buffer to the AudioTrack object
    		audioTrack.write(music, 0, musicLength);

    	} catch (Throwable t) {
    		Log.e("AudioTrack","Playback Failed");
    		t.printStackTrace();
    	}
    }
        
}
